/*
 * $Id: w_trace.c,v 1.1 2006/03/12 08:29:40 slava Exp $
 *
 * Copyright (C) 2000-2006 by Slava Monich
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 *
 *   1.Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer. 
 *   2.Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer 
 *     in the documentation and/or other materials provided with the 
 *     distribution. 
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING 
 * IN ANY WAY OUT OF THE USE OR INABILITY TO USE THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 *
 * The views and conclusions contained in the software and documentation 
 * are those of the authors and should not be interpreted as representing 
 * any official policies, either expressed or implied.
 */

#include "s_util.h"

/**
 * Trace utilities for Win32 environment
 */

#ifndef _WIN32
#  error "_WIN32 must be defined to compile Win32 code"
#endif

STATIC Bool WIN32_stdout = MayBe;
STATIC Bool WIN32_stderr = MayBe;

/**
 * Enables or disables console trace.
 */
void PRINT_UseConsole(Bool enable)
{
    WIN32_stdout = (enable ? True : MayBe);
    WIN32_stderr = (enable ? True : MayBe);
}

/**
 * Determines if standard output stream is available for the process.
 */
STATIC Bool WIN32_HaveStdout()
{
    if (WIN32_stdout == MayBe) {        
        WIN32_stdout = 
#ifdef _WIN32_WCE
            False;
#else /* !_WIN32_WCE */
            (GetStdHandle(STD_OUTPUT_HANDLE) != INVALID_HANDLE_VALUE);
#endif /* !_WIN32_WCE */
    }
    return WIN32_stdout;
}

/**
 * Determines if standard error stream is available for the process.
 */
STATIC Bool WIN32_HaveStderr()
{
    if (WIN32_stderr == MayBe) {
        WIN32_stderr = 
#ifdef _WIN32_WCE
            False;
#else /* !_WIN32_WCE */
            (GetStdHandle(STD_ERROR_HANDLE) != INVALID_HANDLE_VALUE);
#endif /* !_WIN32_WCE */
    }
    return WIN32_stderr;
}

#if DEBUG
/**
 * dumps formatted message to the debug console
 */
STATIC int WIN32_DebugPrintVa(Str szFormat, va_list va)
{

#ifdef _WIN32_WCE
    static Bool writeDebugLog = True;   /* turns off "debug.log" file */
#endif /* !_WIN32_WCE */

    /* 
     * save last error as some of the routines that we invoke may change 
     * it OutputDebugString is one of them
     */
    ULONG lastError = GetLastError();
    int n = 0;

    /* must use exception handling on Windows CE */
#if defined(_USE_EXCEPTION_HANDLING) || defined(_WIN32_WCE)
    __try 
#endif /* _USE_EXCEPTION_HANDLING || _WIN32_WCE */
    {
        Char szBuffer[1020];    /* limitation of OutputDebugString */
        n = Vsnprintf(szBuffer,COUNT(szBuffer),szFormat,va);

        /* if output string is REALLY long, terminate it with ... */
        if (n >= COUNT(szBuffer)-1 || n < 0) {
            n = COUNT(szBuffer)-1;
            szBuffer[n--] = 0;
            if (szFormat[StrLen(szFormat)-1] == '\n') {
                szBuffer[n--] = '\n';
            }
            szBuffer[n--] = '.'; 
            szBuffer[n--] = '.'; 
            szBuffer[n--] = '.';
            szBuffer[n--] = ' ';
        } else {
            szBuffer[ COUNT(szBuffer)-1 ] = 0;
        }

        /* send it to the debug console */
        OutputDebugString(szBuffer);

#ifdef _WIN32_WCE
        /* under Windows CE, dump it to a file too */
        if (writeDebugLog) {
            FILE* log = _tfopen(TEXT("debuglog.txt"), TEXT("at"));
            if (log) {
                _fputts(szBuffer, log);
                fclose(log);
            }
        }
#endif /* !_WIN32_WCE */
    }
#if defined(_USE_EXCEPTION_HANDLING) || defined(_WIN32_WCE)
    __except(EXCEPTION_EXECUTE_HANDLER) {
        ASSMSG1("EXCEPTION %08lX in WIN32_DebugPrintVa",GetExceptionCode());
    }
#endif /* _USE_EXCEPTION_HANDLING || _WIN32_WCE */

    /* restore last error */
    SetLastError(lastError);
    return 0;
}
#endif /* DEBUG */

/**
 * Dumps formatted string to debug console and stdout if such is available
 */
int PRINT_Stdout(Str format, va_list va)
{
    int n = 0;
    DEBUG_ONLY(n = WIN32_DebugPrintVa(format, va));
    if (!WIN32_HaveStdout()) return n;
    n = Vfprintf(stdout, format, va);
    fflush(stdout);
    return n;
}

/**
 * Dumps formatted string to debug console and stderr if such is available
 */
int PRINT_Stderr(Str format, va_list va)
{
    int n = 0;
    DEBUG_ONLY(n = WIN32_DebugPrintVa(format, va));
    if (!WIN32_HaveStderr()) return n;
    n = Vfprintf(stderr, format, va);
    fflush(stderr);
    return n;
}
